=== modified file 'bzrlib/tests/HttpServer.py'
--- bzrlib/tests/HttpServer.py	2006-11-21 11:29:03 +0000
+++ bzrlib/tests/HttpServer.py	2006-12-01 15:06:29 +0000
@@ -159,7 +159,7 @@
             file = open(path, 'rb')
         except IOError:
             self.send_error(404, "File not found")
-            return None
+            return
 
         file_size = os.fstat(file.fileno())[6]
         tail, ranges = self.parse_ranges(ranges_header_value)
@@ -176,12 +176,13 @@
                     ranges_valid = False
                     break
         if not ranges_valid:
-            # RFC2616 14-16 says that invalid Range headers
-            # should be ignored and in that case, the whole file
-            # should be returned as if no Range header was
-            # present
-            file.close() # Will be reopened by the following call
-            return SimpleHTTPRequestHandler.do_GET(self)
+            # RFC2616 14-16 says that invalid Range headers could
+            # be ignored and in that case, the whole file should
+            # be returned as if no Range header was present. Or
+            # that the server should returns a 416 error.
+            file.close()
+            self.send_error(416, "Requested range not satisfiable")
+            return
 
         if len(ranges) == 1:
             (start, end) = ranges[0]

=== modified file 'bzrlib/tests/test_http.py'
--- bzrlib/tests/test_http.py	2006-11-21 01:08:52 +0000
+++ bzrlib/tests/test_http.py	2006-12-01 15:06:29 +0000
@@ -566,18 +566,18 @@
         self.assertEqual(l[2], (0, '0'))
         self.assertEqual(l[3], (3, '34'))
 
-    def test_readv_short_read(self):
+    def test_readv_invalid_ranges(self):
         server = self.get_readonly_server()
         t = self._transport(server.get_url())
 
         # This is intentionally reading off the end of the file
         # since we are sure that it cannot get there
-        self.assertListRaises((errors.ShortReadvError, AssertionError),
+        self.assertListRaises((errors.InvalidRange, errors.ShortReadvError,),
                               t.readv, 'a', [(1,1), (8,10)])
 
         # This is trying to seek past the end of the file, it should
         # also raise a special error
-        self.assertListRaises(errors.ShortReadvError,
+        self.assertListRaises((errors.InvalidRange, errors.ShortReadvError,),
                               t.readv, 'a', [(12,2)])
 
 

=== modified file 'bzrlib/tests/test_transport_implementations.py'
--- bzrlib/tests/test_transport_implementations.py	2006-11-21 01:08:52 +0000
+++ bzrlib/tests/test_transport_implementations.py	2006-12-01 15:06:29 +0000
@@ -1327,10 +1327,12 @@
 
         # This is intentionally reading off the end of the file
         # since we are sure that it cannot get there
-        self.assertListRaises((errors.ShortReadvError, AssertionError),
+        self.assertListRaises((errors.ShortReadvError, errors.InvalidRange,
+                               # Can be raised by paramiko
+                               AssertionError),
                               transport.readv, 'a', [(1,1), (8,10)])
 
         # This is trying to seek past the end of the file, it should
         # also raise a special error
-        self.assertListRaises(errors.ShortReadvError,
+        self.assertListRaises((errors.ShortReadvError, errors.InvalidRange),
                               transport.readv, 'a', [(12,2)])

=== modified file 'bzrlib/transport/http/__init__.py'
--- bzrlib/transport/http/__init__.py	2006-10-17 13:58:01 +0000
+++ bzrlib/transport/http/__init__.py	2006-12-01 15:06:29 +0000
@@ -251,6 +251,32 @@
         """
         return self
 
+    def _retry_get(self, relpath, ranges, exception):
+        """A GET request have failed, let's retry with a simpler request."""
+
+        try_again = False
+        # The server does not gives us enough data or
+        # bogus-looking result, let's try again with
+        # a simpler request if possible.
+        if self._range_hint == 'multi':
+            self._range_hint = 'single'
+            mutter('Retry %s with single range request' % relpath)
+            try_again = True
+        elif self._range_hint == 'single':
+            self._range_hint = None
+            mutter('Retry %s without ranges' % relpath)
+            try_again = True
+        if try_again:
+            # Note that since the offsets and the ranges may not
+            # be in the same order we dont't try to calculate a
+            # restricted single range encompassing unprocessed
+            # offsets.
+            code, f = self._get(relpath, ranges)
+            return try_again, code, f
+        else:
+            # We tried all the tricks, nothing worked
+            raise exception
+
     def readv(self, relpath, offsets):
         """Get parts of the file at the given relative path.
 
@@ -260,7 +286,15 @@
         ranges = self.offsets_to_ranges(offsets)
         mutter('http readv of %s collapsed %s offsets => %s',
                 relpath, len(offsets), ranges)
-        code, f = self._get(relpath, ranges)
+
+        try_again = True
+        while try_again:
+            try_again = False
+            try:
+                code, f = self._get(relpath, ranges)
+            except (errors.InvalidRange, errors.ShortReadvError), exception:
+                try_again, code, f = self._retry_get(relpath, ranges, exception)
+
         for start, size in offsets:
             try_again = True
             while try_again:
@@ -272,31 +306,13 @@
                     if len(data) != size:
                         raise errors.ShortReadvError(relpath, start, size,
                                                      actual=len(data))
-                except (errors.InvalidRange, errors.ShortReadvError):
-                    # The server does not gives us enough data or
-                    # bogus-looking result, let's try again with
-                    # a simpler request if possible.
-                    if self._range_hint == 'multi':
-                        self._range_hint = 'single'
-                        mutter('Retry %s with single range request' % relpath)
-                        try_again = True
-                    elif self._range_hint == 'single':
-                        self._range_hint = None
-                        mutter('Retry %s without ranges' % relpath)
-                        try_again = True
-                    if try_again:
-                        # Note that since the offsets and the
-                        # ranges may not be in the same order we
-                        # dont't try to calculate a restricted
-                        # single range encompassing unprocessed
-                        # offsets. Note that we replace 'f' here
-                        # and that it may need cleaning one day
-                        # before being thrown that way.
-                        code, f = self._get(relpath, ranges)
-                    else:
-                        # We tried all the tricks, nothing worked
-                        raise
-
+                except (errors.InvalidRange, errors.ShortReadvError), exception:
+                    # Note that we replace 'f' here and that it
+                    # may need cleaning one day before being
+                    # thrown that way.
+                    try_again, code, f = self._retry_get(relpath, ranges,
+                                                         exception)
+            # After one or more tries, we get the data.
             yield start, data
 
     @staticmethod

=== modified file 'bzrlib/transport/http/_pycurl.py'
--- bzrlib/transport/http/_pycurl.py	2006-11-30 17:49:47 +0000
+++ bzrlib/transport/http/_pycurl.py	2006-12-01 15:06:29 +0000
@@ -253,6 +253,14 @@
                         _pycurl_errors.CURLE_COULDNT_RESOLVE_PROXY):
                 raise ConnectionError('curl connection error (%s)\non %s'
                               % (e[1], url))
+            elif e[0] == _pycurl_errors.CURLE_PARTIAL_FILE:
+                # Pycurl itself have detected a short read, we do
+                # not have all the informations for the
+                # ShortReadvError, but that should be enough
+                raise errors.ShortReadvError(url,
+                                             offset='unknown', length='unknown',
+                                             actual='unknown',
+                                             extra='Server aborted the request')
             # jam 20060713 The code didn't use to re-raise the exception here
             # but that seemed bogus
             raise

=== modified file 'bzrlib/transport/http/_urllib2_wrappers.py'
--- bzrlib/transport/http/_urllib2_wrappers.py	2006-11-21 01:08:52 +0000
+++ bzrlib/transport/http/_urllib2_wrappers.py	2006-12-01 15:06:29 +0000
@@ -655,6 +655,11 @@
                                                     hdrs, fp))
         elif code == 403:
             raise errors.TransportError('Server refuses to fullfil the request')
+        elif code == 416:
+            # We don't know which, but one of the ranges we
+            # specified was wrong. So we raise with 0 for a lack
+            # of a better magic value.
+            raise errors.InvalidRange(req.get_full_url(),0)
         else:
             # TODO: A test is needed to exercise that code path
             raise errors.InvalidHttpResponse(req.get_full_url(),

=== modified file 'bzrlib/transport/http/response.py'
--- bzrlib/transport/http/response.py	2006-10-16 01:50:48 +0000
+++ bzrlib/transport/http/response.py	2006-12-01 15:06:29 +0000
@@ -279,6 +279,11 @@
         return data
     elif code == 404:
         raise errors.NoSuchFile(url)
+    elif code == 416:
+        # We don't know which, but one the range we specified
+        # was wrong. So we raise with 0 for a lack of a
+        # better magic value.
+        raise errors.InvalidRange(url,0)
 
     # TODO: jam 20060713 Properly handle redirects (302 Found, etc)
     #       The '_get' code says to follow redirects, we probably 

